//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/InclinationAnglesEditor.cpp
//! @brief     Defines class InclinationAnglesEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/InclinationAnglesEditor.h"
#include "GUI/Model/Device/PointwiseAxisItem.h"
#include "GUI/Model/Device/SpecularBeamInclinationItem.h"
#include "GUI/View/Instrument/DistributionEditor.h"
#include "GUI/View/Instrument/DistributionPlot.h"
#include "GUI/View/Instrument/SphericalAxisEditor.h"
#include "GUI/View/Tool/GroupBoxCollapser.h"
#include <QComboBox>
#include <QFormLayout>

InclinationAnglesEditor::InclinationAnglesEditor(QWidget* parent, SpecularBeamInclinationItem* item)
    : QGroupBox("Inclination angles [deg]", parent)
    , m_item(item)
{
    setProperty("subgroup", true); // for stylesheet addressing

    auto* hLayout = new QHBoxLayout(this);
    hLayout->setSpacing(50);
    auto* gform = new QFormLayout;
    gform->setSpacing(6);
    hLayout->addLayout(gform);

    m_selector =
        new DistributionSelector(std::nullopt, GUI::ID::Distributions::Symmetric, this, m_item);
    gform->addRow(m_selector);

    auto* typeComboBox = new QComboBox(this);
    typeComboBox->addItem("Uniform axis");
    typeComboBox->addItem("Non-uniform axis");
    gform->addRow("Axis type:", typeComboBox);
    connect(typeComboBox, qOverload<int>(&QComboBox::currentIndexChanged), this,
            &InclinationAnglesEditor::onAxisTypeSelected);

    m_form = new SphericalAxisForm(gform, this);

    const int idx = m_item->pointwiseAlphaAxisSelected() ? 1 : 0;
    ASSERT(idx != -1);
    if (idx != typeComboBox->currentIndex())
        typeComboBox->setCurrentIndex(idx);
    else
        onAxisTypeSelected(idx); // trigger update manually
    typeComboBox->setEnabled(m_item->pointwiseAlphaAxisDefined());

    m_plot = new DistributionPlot(this);
    m_plot->setFixedSize(280, 170);
    m_plot->setShowMouseCoords(false);
    hLayout->addWidget(m_plot);
    hLayout->setAlignment(m_plot, Qt::AlignTop);
    hLayout->addStretch(1);

    GroupBoxCollapser::installIntoGroupBox(this);

    connect(m_selector, &DistributionSelector::distributionChanged, this,
            &InclinationAnglesEditor::dataChanged);
    connect(m_selector, &DistributionSelector::distributionChanged, this,
            &InclinationAnglesEditor::updatePlot);
    connect(m_form, &SphericalAxisForm::dataChanged, this, &InclinationAnglesEditor::dataChanged);

    updatePlot();
}

void InclinationAnglesEditor::updateIndicators()
{
    m_form->updateData();
}

void InclinationAnglesEditor::onAxisTypeSelected(int index)
{
    if (m_item) {
        if (index == 0 && m_item->pointwiseAlphaAxisSelected()) {
            m_item->selectUniformAxis();
            emit dataChanged();
        } else if (index == 1 && !m_item->pointwiseAlphaAxisSelected()) {
            m_item->selectPointwiseAxis();
            emit dataChanged();
        }

        m_form->setAxisItem(m_item->alphaAxis());
        m_form->setEnabled(index == 0);
    }
}

void InclinationAnglesEditor::updatePlot()
{
    auto* d = m_selector->item()->currentDistribution();
    m_plot->setVisible(!dynamic_cast<const DistributionNoneItem*>(d));
    m_plot->setDistItem(d);
    m_plot->plotItem();
}
