//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/SampleDesigner/MesocrystalForm.cpp
//! @brief     Implements class MesocrystalForm
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/SampleDesigner/MesocrystalForm.h"
#include "GUI/Model/Sample/FormFactorItems.h"
#include "GUI/Model/Sample/MesocrystalItem.h"
#include "GUI/Model/Sample/ParticleItem.h"
#include "GUI/Util/ActionFactory.h"
#include "GUI/View/SampleDesigner/FormLayouter.h"
#include "GUI/View/Tool/GroupBoxCollapser.h"

#include <QAction>

MesocrystalForm::MesocrystalForm(QWidget* parent, MesocrystalItem* item, SampleEditorController* ec,
                                 bool allowRemove)
    : QGroupBox(parent)
    , m_item(item)
    , m_ec(ec)
{
    setTitle("Meso crystal");
    setObjectName("Mesocrystal");
    FormLayouter layouter(this, ec);
    m_layout = layouter.layout();
    layouter.setContentsMargins(30, 6, 0, 0);
    layouter.addVector(item->position(), false);
    layouter.addSelection(item->rotation().m_descriptor);
    layouter.addValue(item->abundance());
    layouter.addVector(item->vectorA(), false);
    layouter.addVector(item->vectorB(), false);
    layouter.addVector(item->vectorC(), false);

    layouter.addSelection(item->outerShape().m_descriptor);

    m_basisCombo = createBasisCombo(this, item->basisParticle());
    connect(m_basisCombo, QOverload<int>::of(&QComboBox::currentIndexChanged), this,
            &MesocrystalForm::onBasisComboChanged);
    m_rowOfBasisTypeCombo = layouter.addRow("Basis type", m_basisCombo);
    createBasisWidgets();

    auto* collapser = GroupBoxCollapser::installIntoGroupBox(this);

    auto* showInRealspaceAction = ActionFactory::createShowInRealspaceAction(
        this, "meso crystal", [=] { ec->requestViewInRealspace(item); });
    collapser->addAction(showInRealspaceAction);

    if (allowRemove) {
        m_removeAction = ActionFactory::createRemoveAction(this, "meso crystal",
                                                           [=] { ec->removeParticle(item); });
        collapser->addAction(m_removeAction);
    }
}

QComboBox* MesocrystalForm::createBasisCombo(QWidget* parent, ItemWithParticles* current)
{
    auto* combo = new QComboBox(parent);
    WheelEventEater::install(combo);
    combo->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);

    uint32_t currentData = 0;
    for (auto type : FormFactorItemCatalog::types()) {
        const auto ui = FormFactorItemCatalog::uiInfo(type);
        combo->addItem(QIcon(ui.iconPath), ui.menuEntry, static_cast<uint32_t>(type));
        if (auto* p = dynamic_cast<ParticleItem*>(current))
            if (FormFactorItemCatalog::type(p->formfactor()) == type)
                currentData = static_cast<uint32_t>(type);
    }
    for (auto type : ItemWithParticlesCatalog::assemblyTypes()) {
        const auto ui = ItemWithParticlesCatalog::uiInfo(type);
        combo->addItem(QIcon(ui.iconPath), ui.menuEntry, 1000 + static_cast<uint32_t>(type));
        if (ItemWithParticlesCatalog::type(current) == type)
            currentData = 1000 + static_cast<uint32_t>(type);
    }

    combo->setMaxVisibleItems(combo->count());

    const auto currentIndex = combo->findData(currentData);
    ASSERT(currentIndex >= 0);
    combo->setCurrentIndex(currentIndex);

    return combo;
}

void MesocrystalForm::onBasisComboChanged()
{
    while (m_layout->rowCount() > m_rowOfBasisTypeCombo + 1)
        m_layout->removeRow(m_rowOfBasisTypeCombo + 1);

    const auto currentData = m_basisCombo->currentData().toUInt();
    if (currentData < 1000)
        m_ec->setMesocrystalBasis(this, static_cast<FormFactorItemCatalog::Type>(currentData));
    else
        m_ec->setMesocrystalBasis(this,
                                  static_cast<ItemWithParticlesCatalog::Type>(currentData - 1000));
}

void MesocrystalForm::createBasisWidgets()
{
    if (!m_item->basisParticle())
        return;

    m_layout->addRow(LayerEditorUtils::createWidgetForItemWithParticles(
        this, m_item->basisParticle(), false, m_ec, false));
}

void MesocrystalForm::enableStructureEditing(bool b)
{
    if (m_removeAction)
        m_removeAction->setVisible(b);
}

MesocrystalItem* MesocrystalForm::mesocrystalItem() const
{
    return m_item;
}
